#!/usr/bin/env python
#pylint: skip-file
"""
LocationApi.py
    Copyright 2016 Cisco Systems

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
import sys
import os
import urllib.request, urllib.parse, urllib.error

from .models import *


class LocationApi(object):

    def __init__(self, apiClient):
      self.apiClient = apiClient



    def getAllLocation(self, **kwargs):
        """Retrieves all locations

        Args:

            scope, str: Authorization Scope for RBAC (required)



        Returns: LocationNIOListResult
        """

        allParams = ['scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getAllLocation" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/location'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'LocationNIOListResult')
        return responseObject




    def updateLocation(self, **kwargs):
        """Updates location

        Args:

            locationNIO, LocationNIO: Attributes of a location (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: TaskIdResult
        """

        allParams = ['locationNIO', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method updateLocation" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/location'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'PUT'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']







        if ('locationNIO' in params):
            bodyParam = params['locationNIO']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def addLocation(self, **kwargs):
        """Creates location

        Args:

            locationNIO, LocationNIO: Attributes of a location (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: TaskIdResult
        """

        allParams = ['locationNIO', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method addLocation" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/location'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'POST'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']







        if ('locationNIO' in params):
            bodyParam = params['locationNIO']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def getLocationCount(self, **kwargs):
        """Retrieves location count

        Args:

            scope, str: Authorization Scope for RBAC (required)



        Returns: CountResult
        """

        allParams = ['scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getLocationCount" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/location/count'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'CountResult')
        return responseObject




    def getLocationByName(self, **kwargs):
        """Retrieves location by location name

        Args:

            location-name, str: Location name (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: LocationNIOResult
        """

        allParams = ['location-name', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getLocationByName" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/location/location-name/{location-name}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('location-name' in params):
            replacement = str(self.apiClient.toPathValue(params['location-name']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'location-name' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'LocationNIOResult')
        return responseObject




    def updateLocationTag(self, **kwargs):
        """Associate a tag to a location

        Args:

            tagNIO, TagNIO: TagNIO with tag ID and location ID (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: TaskIdResult
        """

        allParams = ['tagNIO', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method updateLocationTag" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/location/tag'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'POST'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']







        if ('tagNIO' in params):
            bodyParam = params['tagNIO']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def getLocationById(self, **kwargs):
        """Retrieves location by ID

        Args:

            id, str: Location ID (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: LocationNIOResult
        """

        allParams = ['id', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getLocationById" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/location/{id}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('id' in params):
            replacement = str(self.apiClient.toPathValue(params['id']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'id' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'LocationNIOResult')
        return responseObject




    def deleteLocationById(self, **kwargs):
        """Deletes location by ID

        Args:

            id, str: Location ID (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: TaskIdResult
        """

        allParams = ['id', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method deleteLocationById" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/location/{id}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'DELETE'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('id' in params):
            replacement = str(self.apiClient.toPathValue(params['id']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'id' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def getLocationByRange(self, **kwargs):
        """Retrieves location range

        Args:

            startIndex, int: Start index (required)


            recordsToReturn, int: Number of records to return (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: LocationNIOListResult
        """

        allParams = ['startIndex', 'recordsToReturn', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getLocationByRange" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/location/{startIndex}/{recordsToReturn}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('startIndex' in params):
            replacement = str(self.apiClient.toPathValue(params['startIndex']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'startIndex' + '}',
                                                replacement)

        if ('recordsToReturn' in params):
            replacement = str(self.apiClient.toPathValue(params['recordsToReturn']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'recordsToReturn' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'LocationNIOListResult')
        return responseObject




    def getLocationByTag(self, **kwargs):
        """Retrieves location by tag

        Args:

            tagId, str: Tag ID (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: LocationNIOListResult
        """

        allParams = ['tagId', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getLocationByTag" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/tag/{tagId}/location'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('tagId' in params):
            replacement = str(self.apiClient.toPathValue(params['tagId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'tagId' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'LocationNIOListResult')
        return responseObject









#!/usr/bin/env python
#pylint: skip-file
"""
TaskApi.py
    Copyright 2016 Cisco Systems

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
import sys
import os
import urllib.request, urllib.parse, urllib.error

from .models import *


class TaskApi(object):

    def __init__(self, apiClient):
      self.apiClient = apiClient



    def getTaskByRange(self, **kwargs):
        """getTaskByRange

        Args:

            offset, int: Index (required)


            limit, int: The maximum value of {limit} supported is 500. &lt;br/&gt; Base 1 indexing for {limit}, minimum value is 1 (required)



        Returns: TaskDTOListResponse
        """

        allParams = ['offset', 'limit']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getTaskByRange" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/task/{offset}/{limit}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'






        if ('offset' in params):
            replacement = str(self.apiClient.toPathValue(params['offset']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'offset' + '}',
                                                replacement)

        if ('limit' in params):
            replacement = str(self.apiClient.toPathValue(params['limit']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'limit' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskDTOListResponse')
        return responseObject




    def getTask(self, **kwargs):
        """getTruststoreFileCount

        Args:

            taskId, str: UUID of the Task (required)



        Returns: TaskDTOResponse
        """

        allParams = ['taskId']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getTask" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/task/{taskId}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'






        if ('taskId' in params):
            replacement = str(self.apiClient.toPathValue(params['taskId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'taskId' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskDTOResponse')
        return responseObject




    def getTaskTree(self, **kwargs):
        """Get Task Tree

        Args:

            taskId, str: UUID of the Task (required)



        Returns: TaskDTOListResponse
        """

        allParams = ['taskId']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getTaskTree" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/task/{taskId}/tree'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'






        if ('taskId' in params):
            replacement = str(self.apiClient.toPathValue(params['taskId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'taskId' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskDTOListResponse')
        return responseObject









"""
Copyright 2016 Cisco Systems

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import time
try:
    from uniq.common import logger
    logger = logger.log
except:
    import logging
    logger = logging.getLogger("uniq")
from uniq.utils import utils


class Services(object):
    """ Services base class"""

    DEFAULT_TIMEOUT = 60  # seconds
    DEFAULT_POLL_FREQUENCY = 1 # seconds

    def __init__(self, nb_api, *args, **kwargs):
        """ Initializer makes available the nb_api client and logger to objects of this class.

        Args:
            nb_api (NbClientManager): instance of NbClientManager
        """

        super().__init__(*args, **kwargs)

        self.nb_api = nb_api
        self.log = logger
        self.utils = utils

    def serialize(self, model):
        """ Serializes any service's model object into dictionary.

        Args:
            model (swagger model object): object of swagger model class

        Returns:
            a serialized dict of the swagger model object
        """

        self.log.info("Serializing swagger model object: {0}".format(model.__class__.__name__))
        return self.nb_api.serialize(model)

    def handle_task(self, task, task_status=None, timeout=DEFAULT_TIMEOUT,
                    poll_frequency=DEFAULT_POLL_FREQUENCY):
        """ Returns the task response for a task on completion

        By default it will only wait for the task to complete, it can be made to wait for a
        particular state (Success/Failure), depending on the argument value of task_status.

        Args:
            task (TaskIdResult): an object of the task response
            task_status (str): task status to wait for (Success/Failure). Default is None
            timeout (float): Number of seconds before timing out.
            poll_frequency (float): Sleep interval between calls.
        """

        if task_status:
            if task_status.lower() == "success":
                return self.nb_api.task_util.wait_for_task_success(task, timeout, poll_frequency)
            elif task_status.lower() == "failure":
                return self.nb_api.task_util.wait_for_task_failure(task, timeout, poll_frequency)
            else:
                raise Exception('Unrecognized task status "{}".'.format(task_status))
        return self.nb_api.task_util.wait_for_task_complete(task, timeout, poll_frequency)

    def waits_until(self, method, timeout=DEFAULT_TIMEOUT, poll_frequency=DEFAULT_POLL_FREQUENCY,
                    ignored_exceptions=None, message='', *args, **kwargs):
        """ Calls the method provided until the return value is not False.

        Args:
            method (function): method to call for checking status.
            timeout (float): Number of seconds before timing out.
            poll_frequency (float): Sleep interval between calls.
            ignored_exceptions (list(Exception)): Exceptions to ignore.
            message (str): Message shows in TimeoutException.
            args (list): Arguments of method.
            kwargs (dict): Dictionary of arguments of method.

        Returns:
            Value of method if value is not False.

        Raises:
            TimeoutException if value of method is still False when time's up.
        """

        return self._waits_until(method, False, timeout, poll_frequency, ignored_exceptions, message,
                                 *args, **kwargs)

    def waits_until_not(self, method, timeout=DEFAULT_TIMEOUT,
                        poll_frequency=DEFAULT_POLL_FREQUENCY, ignored_exceptions=None,
                        message='', *args, **kwargs):
        """ Calls the method provided until the return value is False.

        Args:
            method (function): method to call for checking status.
            timeout (float): Number of seconds before timing out.
            poll_frequency (float): Sleep interval between calls.
            ignored_exceptions (list(Exception)): Exceptions to ignore.
            message (str): Message shows in TimeoutException.
            args (list): Arguments of method.
            kwargs (dict): Dictionary of arguments of method.

        Returns:
            False if value of method turns to False in time.

        Raises:
            TimeoutException if value of method is not False when time's up.
        """

        self._waits_until(method, True, timeout, poll_frequency, ignored_exceptions, message,
                          *args, **kwargs)
        return False

    def _waits_until(self, method, wait_false=False, timeout=DEFAULT_TIMEOUT,
                     poll_frequency=DEFAULT_POLL_FREQUENCY,
                     ignored_exceptions=None, message='', *args, **kwargs):
        """ Calls the method provided until the return value is not False.

        Args:
            method (function): method to call for checking status.
            wait_false (bool): true if wait method return value is not False, otherwise, wait return
                               value become False.
            timeout (float): Number of seconds before timing out.
            poll_frequency (float): Sleep interval between calls.
            ignored_exceptions (list(Exception)): Exceptions to ignore.
            message (str): Message shows in TimeoutException.
            args (list): Arguments of method.
            kwargs (dict): Dictionary of arguments of method.

        Returns:
            Value of method if value is not False. False if timeout.

        Raises:
            TimeoutException if value of method is still False when time's up.
        """

        if ignored_exceptions:
            ignored_exceptions = tuple(ignored_exceptions)
        else:
            ignored_exceptions = tuple()

        if not message:
            message = ("Timeout, didn't get any non-False value from method {} in {} seconds."
                       .format(method, timeout))

        start_time = time.time()
        self.log.debug("Begins at {0}".format(start_time))

        end_time = start_time + timeout
        while True:
            try:
                value = method(*args, **kwargs)
                if wait_false:
                    value = not value
                if value is not False:
                    self.log.debug("Waited for {0} seconds.".format(time.time() - start_time))
                    return value
            except ignored_exceptions as exc:
                self.log.debug("Ignore exception: {0}:{1}".format(type(exc), exc))
            except:
                raise
            time.sleep(poll_frequency)
            if time.time() > end_time:
                break
        raise TimeoutException(message)





"""
Copyright 2016 Cisco Systems

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import os
import importlib
import json
import ssl
import sys
import datetime
import re
import itertools

from uniq.apis.client_manager import ClientManager
from uniq.apis.nb.clients.api_client import ApiClient
from uniq.apis.exceptions import UniqException
from uniq.apis.nb.services.task.task import Task

# Python would use the system provided certificate database on all platforms.
# Failure to locate such a database would be an error, and users would need to explicitly
# specify a location to fix it. This will be achieved by adding a new
# ssl._create_default_https_context function. http.client can then replace its usage of
# ssl._create_stdlib_context with the ssl._create_default_https_context.
# Reference: https://www.python.org/dev/peps/pep-0476/#technical-details
try:
    ssl._create_default_https_context = ssl._create_unverified_context
except Exception:
    pass


class NbClientManager(ClientManager):
    """ Northbound Client manager to interact with Swagger's API Client. """

    CLIENTS_PATH = os.path.join(os.path.split(os.path.realpath(__file__))[0], "clients")
    IMPORT_PATH = "uniq.apis.nb.clients"
    MODELS_DIR = "models"
    CLIENT_DIR_PATTERN = "_client"
    API_FILE_PATTERN = "Api.py"
    NULLABLE_TYPES = ['str', 'bool'] # 'None', False

    VERSION_V2 = "V2"

    def __init__(self, server, username, password, version="v1", connect=True):
        """ Object initializer

        Initializer also aunthenticates using the credentials, and stores the generated
        authentication ticket.

        Args:
            server (str): cluster server name (routable DNS addess or ip)
            username (str): user name to authenticate with
            password (str): password to authenticate with
            version (str): version of the API to be used
            connect (bool): flag to authenticate and establish swagger client
        """

        self.version = version
        base_url = "/api/" + self.version
        protocol = "https"

        super(NbClientManager, self).__init__(server, username, password, base_url,
                                              protocol=protocol)

        self.default_headers = {"Content-Type": "application/json"}

        self.api_dict = self.__load_apis(self.CLIENTS_PATH, self.IMPORT_PATH)
        self.duplicate_models = {}
        self.conflicting_models = {}
        self.model_dict = self.__load_models(self.CLIENTS_PATH, self.IMPORT_PATH)
        self.custom_api_client_path = []
        self.similar_api_dict = {}

        if connect:
            self.connect()
        self.setup_api()
        self.cookie = None

    def __repr__(self):
        """ Overrides the default object representation to display the object attributes. """

        return "[APIC: <server:{}> <username:{}> <password:{}> <version:{}>]".format(self.server,
                                                                                     self.username,
                                                                                     self.password,
                                                                                     self.version)

    def connect(self):
        """ Generates a new ticket and establishes a fresh swagger client. """

        self.log.info("Connecting to the Apic-em north bound API client.")
        self.authenticate()

    def disconnect(self):
        """ Deletes the generated ticket and effectively disconneting the user. """

        self.log.info("Disconnecting the Apic-em north bound API client.")

    def __load_models(self, clients_path, import_path=None, model_dict=None):
        """ Find all models files and their import path.

        Args:
            clients_path (str): path to all api clients.
            import_path (str): prefix of import path.

        Returns:
            A dict of model's name and its import path.
            Example:
            model_dict = {
                'Timestamp':
                    'uniq.apis.nb.clients.inventory_manager_client.client.models.Timestamp'
            }
        """

        if model_dict is None:
            model_dict = {}

        clients_dirs = os.listdir(clients_path)
        for client_dir in clients_dirs:
            if client_dir.endswith(self.CLIENT_DIR_PATTERN):
                models_dir = os.path.join(clients_path, client_dir, self.MODELS_DIR)
                model_files = os.listdir(models_dir)
                for model_file in model_files:
                    if model_file != "__init__.py" and model_file != "__pycache__":
                        model_file = model_file.replace(".py", "")
                        if import_path:
                            import_str = ".".join([self.IMPORT_PATH, client_dir, "models",
                                                   model_file])
                        else:
                            import_str = ".".join([client_dir, "models", model_file])

                        # Identify if the model name already exists.
                        if model_file in model_dict:
                            if model_file not in self.duplicate_models:
                                self.duplicate_models[model_file] = [model_dict[model_file]]
                            self.duplicate_models[model_file].append(import_str)
                        else:
                            model_dict[model_file] = import_str
        return model_dict

    def __load_apis(self, clients_path, import_path=None):
        """ Find all api files and their import path.

        Args:
            clients_path (str): path to all api clients.
            import_path (str): prefix of import path.

        Returns:
            A dict of api's name and its import path.
            Example:
            api_dict = {
                'DiscoveryApi':
                    'uniq.apis.nb.clients.inventory_manager_client.client.DiscoveryApi'
            }
        """

        api_dict = {}

        clients_dirs = os.listdir(clients_path)
        for client_dir in clients_dirs:
            if client_dir.endswith(self.CLIENT_DIR_PATTERN):
                api_files_dir = os.path.join(clients_path, client_dir)
                api_files = os.listdir(api_files_dir)
                for api_file in api_files:
                    if api_file.endswith(self.API_FILE_PATTERN):
                        api_file = api_file.replace(".py", "")
                        if import_path:
                            import_str = ".".join([self.IMPORT_PATH, client_dir, api_file])
                        else:
                            import_str = ".".join([client_dir, api_file])
                        api_dict[api_file] = import_str
        return api_dict

    def __get_api_instance(self, api_name):
        """ Get api instance by its name.

        Note:
            Only be able to get swagger auto-generated api client instance.

        Args:
            api_name (str): api name.

        Returns:
            An instance of api_name class
        """

        if api_name not in self.api_dict:
            raise Exception("Can't find api \"{}\"".format(api_name))
        if isinstance(self.api_dict[api_name], ApiClient):
            return self.api_dict[api_name]

        # Convert all api clients to uniq.nb.apis.clients.ApiClient.
        api_module = importlib.import_module(self.api_dict[api_name])
        api_class = getattr(api_module, api_name)

        if api_name.startswith(self.VERSION_V2):
            default_version = 'v2'
            version_client_map = {
                'v2': api_class
            }
        else:
            default_version = 'v1'
            version_client_map = {
                'v1': api_class
            }

        return ApiClient(self, version_client_map, default_version=default_version)

    def __deal_with_multiple_version(self):
        """ Create ApiClient object to deal with multiple version.

        If an endpoint has both v1 and v2 version of apis, this method will create an instance
        of ApiClient to handle multiple versions.
        If an endpoint has only v1 or v2 version of api, this method won't do anything.
        """

        for api in list(self.api_dict.keys()):
            if api.startswith(self.VERSION_V2):
                v1_api_name = api.replace(self.VERSION_V2, "")
                if v1_api_name in self.api_dict:
                    v2_module = importlib.import_module(self.api_dict[api])
                    v1_module = importlib.import_module(self.api_dict[v1_api_name])
                    self.__update_similar_api_dict(v1_api_name, api)
                    self.api_dict.pop(api)
                    self.api_dict.pop(v1_api_name)
                    version_client_map = {
                        'v1': getattr(v1_module, v1_api_name),
                        'v2': getattr(v2_module, api)
                    }
                    self.log.debug("Found similar apis: {}".format(self.similar_api_dict))
                    if v1_api_name in self.similar_api_dict:
                        self.api_dict[v1_api_name] = ApiClient(
                            self, version_client_map,
                            default_version='v1',
                            similar_api_map_list=self.similar_api_dict[v1_api_name])
                    else:
                        self.api_dict[v1_api_name] = ApiClient(self, version_client_map,
                                                               default_version='v1')

    def __deal_with_duplicate_models(self):
        """ Use most compatible models if duplicate models exist. """

        for model, imports in self.duplicate_models.items():
            self.model_dict[model] = self.__get_most_compatible_model(model, imports)

    def __dicts_are_compatible(self, dict_a, dict_b):
        """ Check if two dicts are compatible.

        Args:
            dict_a (dict): first dict
            dict_b (dict): second dict

        Note:
            Compatible means dict_a and dict_b must obey:
            1. dict_a and dict_b are identical. (All key value pairs are same.)
            2. dict_a contains dict_b (All key value pairs in dict_b also in dict_a)
            3. dict_b contains dict_a (All key value pairs in dict_a also in dict_b)

        Examples:
            >>> a = {1:1, 2:2}
            >>> b = {1:1, 2:3}
            >>> c = {1:1}

            >>> # a[2] != b[2]
            >>> dicts_are_compatible(a, b)
            False

            >>> # a contains c, which can be checked by a.items() > c.items()
            >>> dicts_are_compatible(a, c)
            True

        Returns True if two dicts are compatible.
        """

        dict_a_contains_b = dict_a.items() >= dict_b.items()
        dict_b_contains_a = dict_a.items() <= dict_b.items()

        return dict_a_contains_b or dict_b_contains_a

    def __get_most_compatible_model(self, model_name, import_paths):
        """ Get the most compatible model for a model name.

        Args:
            model_name (str): name of model.
            import_paths (list[str]): list of model paths for same model name.

        Note:
            What is most compatible?
            Attribute 'swaggerTypes' and 'attributeMap' are two attributes which decide the model.
            If those two dicts are compatible in two models(see definition of compatible in method
            '__dicts_are_compatible'), we say these two models are compatible. And if one's
            swaggerTypes contains more keys than another, we say the model is more compatible than
            another.

        Returns:
            Import path (str) of the most compatible model.
        """

        model_objs = [getattr(importlib.import_module(import_path), model_name)()
                      for import_path in import_paths]

        for obj_a, obj_b in itertools.combinations(model_objs, 2):
            if (self.__dicts_are_compatible(obj_a.swaggerTypes, obj_b.swaggerTypes) and
                self.__dicts_are_compatible(obj_a.attributeMap, obj_b.attributeMap)):
                continue
            else:
                self.log.info("Model {} and {} are not compatible."
                                  .format(obj_a.__class__, obj_b.__class__))
                if model_name not in self.conflicting_models:
                    self.conflicting_models[model_name] = set()
                self.conflicting_models[model_name].add(obj_a.__class__)
                self.conflicting_models[model_name].add(obj_b.__class__)

        if model_name in self.conflicting_models:
            return None
        else:
            index = model_objs.index(max(model_objs, key=lambda obj: obj.swaggerTypes.items()))
            return import_paths[index]

    def __update_similar_api_dict(self, v1_api_name, v2_api_name):
        """ Find similar api in different version apis and update similar api dict.

        Args:
            v1_api_name (str): version 1 api name.
            v2_api_name (str): version 2 api name.
        """

        v1_api_detail = self.read_json_by_module_path(self.api_dict[v1_api_name], "v1")
        v2_api_detail = self.read_json_by_module_path(self.api_dict[v2_api_name], "v2")
        if v1_api_detail and v2_api_detail:
            for v1_api in v1_api_detail["apis"]:
                for v2_api in v2_api_detail["apis"]:
                    if (v1_api["path"] == v2_api["path"].replace("/v2", "") and
                        v1_api["operations"][0]["method"] == v2_api["operations"][0]["method"] and
                        v1_api["operations"][0]["nickname"] != v2_api["operations"][0]["nickname"]):
                        if v1_api_name not in self.similar_api_dict:
                            self.similar_api_dict[v1_api_name] = []
                        self.similar_api_dict[v1_api_name].append(
                            {
                                "v1": v1_api["operations"][0]["nickname"],
                                "v2": v2_api["operations"][0]["nickname"]}
                            )

    @staticmethod
    def convert_module_name_to_json_file_name(name):
        """ Convert module name to json file name.

        Example:
            'PolicyApi' -> 'policy.json'
            'V2PolicyApi' -> 'v2-policy.json'

        Args:
            name (str): module name

        Returns:
            converted name (str).
        """

        s1 = re.sub('(.)([A-Z][a-z]+)', r'\1-\2', name)
        return re.sub('([a-z0-9])([A-Z])', r'\1-\2', s1).lower().replace("-api", ".json")

    def read_json_by_module_path(self, module_path, version):
        """ Read api json file.

        Args:
            module_path (str): module path of api.
            version (str): version of api.

        Returns:
            content (dict) of json file.
        """

        for sys_path in self.custom_api_client_path + sys.path:
            module_name = module_path.split(".")[-1]
            json_file_name = NbClientManager.convert_module_name_to_json_file_name(module_name)
            path = os.path.join(os.path.join(sys_path, *module_path.split(".")[:-1]),
                                json_file_name)
            if os.path.isfile(path):
                with open(path) as json_file:
                    content = json.load(json_file)
                return content
        return None

    def setup_api(self):
        """ Creates a swagger api client

            For swagger auto-generated api client, the naming rule is all lower case without ending
            with "api" of its classname.
            For example:
                self.user is an instance of UserApi.

        Args:
            ticket (str): session ticket
        """

        self.log.debug("Initializing the swagger north bound api client and endpoints.")

        self.__deal_with_multiple_version()
        self.__deal_with_duplicate_models()

        for api in self.api_dict:
            if api.endswith("Api"):
                attr_name = api[:-3]
            # If a service has only v2 api, get rid of prefix in client name.
            # For ex: V2ContractApi becomes ContractApi
            if attr_name.startswith(self.VERSION_V2):
                attr_name = attr_name.replace(self.VERSION_V2, "")

            self.add_api(attr_name.lower(), self.__get_api_instance(api))

        # Task Wrapper Library
        self.task_util = Task(self)

    def add_new_apis(self, nb_client_path):
        """ Finds and loads NB clients(their models, api files) from user-defined location.

        Args:
            nb_client_path (str): path to new clients, inside it should contain folders naming
                as apiname_client.

        Notes:
            If there is api file having the same filename and classname as in default api client
            folder, it will override the default one. Same for models.
        """

        nb_client_path = os.path.expanduser(nb_client_path)
        if not os.path.isdir(nb_client_path):
            self.log.error("{} is not a valid directory.".format(nb_client_path))
            raise Exception("{} is not a valid directory.".format(nb_client_path))
        sys.path.append(nb_client_path)
        self.custom_api_client_path.append(nb_client_path)

        self.api_dict.update(self.__load_apis(nb_client_path))
        self.__load_models(nb_client_path, model_dict=self.model_dict)

        self.setup_api()

    def serialize(self, model):
        """ Returns the serialized model

        Args:
            model (model object): Model object that needs to be serialized

        Returns:
            serialized model dict
        """

        return NbClientManager.sanitize_for_serialization(model)

    def authenticate(self):
        """ Generates a new authentication cas_ticket. """

        resource_path = "/ticket"
        data = json.dumps({"username": self.username, "password": self.password})
        response = self.call_api("POST", resource_path, data=data, raise_exception=True,
                                 response_dict=False)
        result_json = json.loads(response.text)
        if 'serviceTicket' in result_json['response'].keys():
            ticket = result_json["response"]["serviceTicket"]
        else:
            #TODO (hikoppu): Handle missing service ticket scenario more gracefully
            ticket = ""
        headers = {"X-Auth-Token": ticket, "X-CSRF-Token": "soon-enabled"}
        self.common_headers = headers
        self.cas_ticket = ticket

    def callAPI(self, resource_path, method, query_params, post_data,
                header_params={}, files=None, stream=False, api_version=None):
        """ Wrapper of call_api to support swagger auto generated code.

        Args:
            resource_path (str): resource_path
            method (str): http method, support "GET", "POST", "PUT", "DELETE"
            query_params (dict): query parameters
            post_data (dict or list): body to post.
            header_params (dict): custom headers.
            files (str): path to file
            stream (bool): return stream object.

        Returns:
            data (str): response of request.

        Raises:
            requests.exceptions.HTTPError if any http error happens.
        """

        headers = self.default_headers.copy()
        if header_params:
            for param, value in header_params.items():
                headers[param] = NbClientManager.sanitize_for_serialization(value)
        if self.cookie:
            headers['Cookie'] = NbClientManager.sanitize_for_serialization(self.cookie)

        data = None
        kwargs = {}

        if query_params:
            kwargs["params"] = query_params

        if method in ['GET']:
            if 'Content-Type' in headers:
                if (headers['Content-Type'] == 'multipart/form-data'
                    or headers['Accept'] == 'application/octet-stream'
                    or headers['Content-Type'] == 'application/octet-stream'):
                    return self.getFile(resource_path=resource_path,
                                        params=query_params,
                                        stream=stream,
                                        headers=headers)
            else:
                pass
        elif method in ['POST', 'PUT', 'DELETE']:
            if post_data or post_data == [] or files:

                if post_data:
                    post_data = NbClientManager.sanitize_for_serialization(post_data)
                if files:
                    files = NbClientManager.sanitize_for_serialization(files)
                    post_data = files

                if 'Content-Type' in headers and headers['Content-Type'] == 'application/json':
                    data = json.dumps(post_data)
                    kwargs["data"] = data.encode('utf-8')
                elif headers['Content-Type'] == 'multipart/form-data':
                    return self.postFile(method=method,
                                         files=post_data,
                                         resource_path=resource_path,
                                         params=query_params,
                                         headers=headers)
                else:
                    kwargs["data"] = post_data

        if method not in ["GET", "POST", "PUT", "DELETE"]:
            raise Exception('Method ' + method + ' is not recognized.')

        response = self.call_api(method, resource_path, headers=headers,
                                 response_dict=False, **kwargs)

        if 'Set-Cookie' in response.headers:
            self.cookie = response.headers['Set-Cookie']
        string = response.text

        if type(string) is bytes:
            string = string.decode()

        try:
            data = json.loads(string)
        except ValueError:  # PUT requests don't return anything
            data = None

        return data

    def postFile(self, method, files, resource_path, headers, params=None):
        """ Posts the files

        Args:
            method (str): http method, support "POST", "PUT"
            files (str or dict): File(s) in the request object
            resource_path (str): URL in the request object
            headers (str): Headers in the request object
            params (dict): key/value pairs in the URL after a '?', Ex: httpbin.org/get?key=val

        Returns:
            data (response): JSON form of the response of the post method

        Raises:
            RequestException if invalid data is posted
            ValueError if the response of the post method cannot be converted to JSON.
        """

        headers.pop('Content-Type')
        files_to_post = {}
        if type(files) == dict:
            for name, filepath in files.items():
                fileobj = open(filepath, 'rb')
                files_to_post[name] = fileobj

        if type(files) == str:
            fileobj = open(files, 'rb')
            files_to_post['fileUpload'] = fileobj

        response = self.call_api(method=method,
                                 resource_path=resource_path,
                                 params=params,
                                 files=files_to_post,
                                 verify=False,
                                 headers=headers,
                                 response_dict=False)

        try:
            data = json.loads(response.text)
        except ValueError as err:
            data = None
            raise err

        return data

    def getFile(self, resource_path, headers, stream, params=None):
        """ Gets the files

        Args:
            resource_path (str): URL in the request object
            headers (str): Headers in the request object
            params (dict): key/value pairs in the URL after a '?', Ex: httpbin.org/get?key=val
            stream (bool): return stream object

        Returns:
            Response of the get method

        Raises:
            RequestException if invalid data is being requested
        """

        response = self.call_api("GET",
                                 resource_path=resource_path,
                                 params=params,
                                 verify=False,
                                 headers=headers,
                                 response_dict=False,
                                 stream=stream)
        return response

    def toPathValue(self, obj):
        """Convert a string or object to a path-friendly value
        Args:
            obj -- object or string value
        Returns:
            string -- quoted value
        """
        if type(obj) == list:
            return ','.join(obj)
        else:
            return str(obj)

    @staticmethod
    def sanitize_for_serialization(obj):
        """
        Sanitize an object for Request.

        If obj is None, return None.
        If obj is str, int, float, bool, return directly.
        If obj is datetime.datetime, datetime.date convert to string in iso8601 format.
        If obj is list, santize each element in the list.
        If obj is dict, return the dict.
        If obj is swagger model, return the properties dict.
        """
        if isinstance(obj, type(None)):
            return None
        elif isinstance(obj, (str, int, float, bool)):
            return obj
        elif isinstance(obj, list):
            return [NbClientManager.sanitize_for_serialization(sub_obj) for sub_obj in obj]
        elif isinstance(obj, (datetime.datetime, datetime.date)):
            return obj.isoformat()
        else:
            if isinstance(obj, dict):
                obj_dict = obj
            else:
                # Convert model obj to dict except attributes `swaggerTypes`, `attributeMap`
                # and attributes which value is not None.
                # Convert attribute name to json key in model definition for
                # request.
                obj_dict = {obj.attributeMap[key]: val
                            for key, val in obj.__dict__.items()
                            if key != 'swaggerTypes' and key != 'attributeMap' and val is not None}
            return {key: NbClientManager.sanitize_for_serialization(val)
                    for (key, val) in obj_dict.items()}

    def deserialize(self, obj, objClass):
        """Derialize a JSON string into an object.

        Args:
            obj -- string or object to be deserialized
            objClass -- class literal for deserialzied object, or string
                of class name
        Returns:
            object -- deserialized object"""

        # Have to accept objClass as string or actual type. Type could be a
        # native Python type, or one of the model classes.
        if type(objClass) == str:
            if 'list[' in objClass:
                match = re.match('list\[(.*)\]', objClass)
                subClass = match.group(1)
                return [self.deserialize(subObj, subClass) for subObj in obj]

            if (objClass in ['int', 'float', 'dict', 'list', 'str', 'bool', 'datetime']):
                objClass = eval(objClass)
            elif objClass in self.conflicting_models:
                raise UniqException(
                    "Multiple conflicting models {} exist {}, please check those models and make "
                    "sure they are compatible.".format(objClass, self.conflicting_models[objClass]))
            elif objClass in self.model_dict: # in auto-generated models
                api_module = importlib.import_module(self.model_dict[objClass])
                objClass = getattr(api_module, objClass)
            else:  # not a native type, must be model class
                objClass = eval(objClass + '.' + objClass)

        if objClass in [int, int, float, dict, list, str, bool]:
            if obj is not None:
                return objClass(obj)
            elif objClass in NbClientManager.NULLABLE_TYPES:
                return objClass(obj)
            else:
                return None
        elif objClass == datetime:
            return self.__parse_string_to_datetime(obj)

        instance = objClass()

        for attr, attrType in instance.swaggerTypes.items():
            if obj is not None and instance.attributeMap[attr] in obj and type(obj) in [list, dict]:
                value = obj[instance.attributeMap[attr]]
                if attrType in ['unicode', 'str', 'int', 'float', 'bool']:
                    attrType = eval(attrType)
                    # print "value_orig: '%s'. Type: %s" % (value, type(value))
                    if (value is None):
                        setattr(instance, attr, None)
                    else:
                        try:
                            value = attrType(value)
                        except UnicodeEncodeError:
                            value = str(value)
                        except TypeError:
                            value = value
                        setattr(instance, attr, value)
                    # print "value: '%s'. Type: %s" % (value, type(value))
                elif (attrType == 'datetime'):
                    setattr(
                        instance, attr, self.__parse_string_to_datetime(value))
                elif (attrType == 'date-time'):
                    # Java TimeStamp
                    setattr(instance, attr, self.deserialize(value, int))
                elif 'list[' in attrType:
                    match = re.match('list\[(.*)\]', attrType)
                    subClass = match.group(1)
                    subValues = []
                    if not value:
                        setattr(instance, attr, None)
                    else:
                        for subValue in value:
                            subValues.append(
                                self.deserialize(subValue, subClass))
                    setattr(instance, attr, subValues)
                elif 'simple_dict' in attrType: #TODO DELETE
                    if not value:
                        setattr(instance, attr, None)
                    else:
                        new_dict = {}
                        for key in list(value.keys()):
                            dicValue = value[key]
                            if ((type(dicValue) is not str) and (type(dicValue) is not str)):
                                assert False, "wrong type for key %s: %s" % (key, type(dicValue))
                            new_dict[key] = dicValue
                        setattr(instance, attr, new_dict)
                else:
                    setattr(instance, attr, self.deserialize(value, attrType))

        return instance

    def __parse_string_to_datetime(self, string):
        """
        Parse datetime in string to datetime.

        The string should be in iso8601 datetime format.
        """
        try:
            from dateutil.parser import parse
            return parse(string)
        except ImportError:
            return string



#!/usr/bin/env python
#pylint: skip-file
"""
PnpdeviceApi.py
    Copyright 2016 Cisco Systems

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
import sys
import os
import urllib.request, urllib.parse, urllib.error

from .models import *


class PnpdeviceApi(object):

    def __init__(self, apiClient):
      self.apiClient = apiClient



    def getPnpDevices(self, **kwargs):
        """Retrieves a list of devices

        Args:

            serialNumber, str: Serial number (required)


            macAddress, str: MAC address (required)


            ipAddress, str: IP address (required)


            productId, str: Product ID (required)


            hostName, str: Host name (required)


            pkiEnabled, str: Device certificate provisioned (true) or not (false) (required)


            provisioningType, str: Provisioning type. Can be one of: new-device-adhoc, new-device-auto-provision, replacement-device (required)


            deviceType, str: Type of device. Currently only AP (Access Point) is supported (required)


            state, str: State of device. UNCLAIMED, IGNORED, PROVISIONED etc. Accepts a list of states (required)


            authStatus, str: Authentication status of device: authenticated, error, unsupported (required)


            startLastStateTransitionTime, str: Start of last state transition time (format: yyyy-MM-dd HH:mm:ss) (required)


            endLastStateTransitionTime, str: End of last state transition time (format: yyyy-MM-dd HH:mm:ss) (required)


            matchDeviceState, bool: Match device state. Boolean to match the given state (true) or not (false) (required)


            deviceMatchesARule, bool: Should device match a rule (true) or not (false) (required)


            sortBy, str: Sort response based on field (required)


            order, str: Order response in ascending (asc) or descending (des) order (required)


            offset, int: Starting index of the response. Minimum value is 1 (required)


            limit, int: Limit the number of responses. Maximum value supported is 500. Minimum value is 1 (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: ZtdDeviceListResult
        """

        allParams = ['serialNumber', 'macAddress', 'ipAddress', 'productId', 'hostName', 'pkiEnabled', 'provisioningType', 'deviceType', 'state', 'authStatus', 'startLastStateTransitionTime', 'endLastStateTransitionTime', 'matchDeviceState', 'deviceMatchesARule', 'sortBy', 'order', 'offset', 'limit', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getPnpDevices" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/pnp-device'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('serialNumber' in params):
            queryParams['serialNumber'] = self.apiClient.toPathValue(params['serialNumber'])

        if ('macAddress' in params):
            queryParams['macAddress'] = self.apiClient.toPathValue(params['macAddress'])

        if ('ipAddress' in params):
            queryParams['ipAddress'] = self.apiClient.toPathValue(params['ipAddress'])

        if ('productId' in params):
            queryParams['productId'] = self.apiClient.toPathValue(params['productId'])

        if ('hostName' in params):
            queryParams['hostName'] = self.apiClient.toPathValue(params['hostName'])

        if ('pkiEnabled' in params):
            queryParams['pkiEnabled'] = self.apiClient.toPathValue(params['pkiEnabled'])

        if ('provisioningType' in params):
            queryParams['provisioningType'] = self.apiClient.toPathValue(params['provisioningType'])

        if ('deviceType' in params):
            queryParams['deviceType'] = self.apiClient.toPathValue(params['deviceType'])

        if ('state' in params):
            queryParams['state'] = self.apiClient.toPathValue(params['state'])

        if ('authStatus' in params):
            queryParams['authStatus'] = self.apiClient.toPathValue(params['authStatus'])

        if ('startLastStateTransitionTime' in params):
            queryParams['startLastStateTransitionTime'] = self.apiClient.toPathValue(params['startLastStateTransitionTime'])

        if ('endLastStateTransitionTime' in params):
            queryParams['endLastStateTransitionTime'] = self.apiClient.toPathValue(params['endLastStateTransitionTime'])

        if ('matchDeviceState' in params):
            queryParams['matchDeviceState'] = self.apiClient.toPathValue(params['matchDeviceState'])

        if ('deviceMatchesARule' in params):
            queryParams['deviceMatchesARule'] = self.apiClient.toPathValue(params['deviceMatchesARule'])

        if ('sortBy' in params):
            queryParams['sortBy'] = self.apiClient.toPathValue(params['sortBy'])

        if ('order' in params):
            queryParams['order'] = self.apiClient.toPathValue(params['order'])

        if ('offset' in params):
            queryParams['offset'] = self.apiClient.toPathValue(params['offset'])

        if ('limit' in params):
            queryParams['limit'] = self.apiClient.toPathValue(params['limit'])



        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'ZtdDeviceListResult')
        return responseObject




    def updatePnpDevice(self, **kwargs):
        """Updates a device with its ID

        Args:

            device, list[ZtdDevice]: PnP Device. The only modifiable properties are 1) state 2) configId and 3) imageId. Following are the only allowed combinations: 1) state = START_PROVISIONING, imageId = &lt;&gt; and/or configId = &lt;&gt;. 2) state = IGNORED 3) state = UNCLAIMED. Any other combinations of user input are rejected.  (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: TaskIdResult
        """

        allParams = ['device', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method updatePnpDevice" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/pnp-device'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'PUT'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']







        if ('device' in params):
            bodyParam = params['device']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def getPnpDeviceHistory(self, **kwargs):
        """Retrieves a device&#39;s history with its serial number

        Args:

            serialNumber, str: Serial number (required)


            order, str: Order response in ascending (asc) or descending (des) order of serial number (required)


            offset, int: Starting index of the response. Minimum value is 1 (required)


            limit, int: Limit the number of responses. Maximum value supported is 500. Minimum value is 1 (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: ZtdHistoryListResult
        """

        allParams = ['serialNumber', 'order', 'offset', 'limit', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getPnpDeviceHistory" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/pnp-device-history'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('serialNumber' in params):
            queryParams['serialNumber'] = self.apiClient.toPathValue(params['serialNumber'])

        if ('order' in params):
            queryParams['order'] = self.apiClient.toPathValue(params['order'])

        if ('offset' in params):
            queryParams['offset'] = self.apiClient.toPathValue(params['offset'])

        if ('limit' in params):
            queryParams['limit'] = self.apiClient.toPathValue(params['limit'])



        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'ZtdHistoryListResult')
        return responseObject




    def getPnpDeviceHistoryCount(self, **kwargs):
        """Retrieves a count of a device&#39;s history with its serial number

        Args:

            serialNumber, str: Serial number (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: CountResult
        """

        allParams = ['serialNumber', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getPnpDeviceHistoryCount" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/pnp-device-history/count'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('serialNumber' in params):
            queryParams['serialNumber'] = self.apiClient.toPathValue(params['serialNumber'])



        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'CountResult')
        return responseObject




    def getPnpDeviceStateMap(self, **kwargs):
        """Retrieves a map of internal and external device states

        Args:

            scope, str: Authorization Scope for RBAC (required)



        Returns: ZtdDeviceStateMapResult
        """

        allParams = ['scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getPnpDeviceStateMap" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/pnp-device-state-map'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'ZtdDeviceStateMapResult')
        return responseObject




    def getPnpDeviceCount(self, **kwargs):
        """Retrieves a count of devices

        Args:

            serialNumber, str: Serial number (required)


            macAddress, str: MAC address (required)


            ipAddress, str: IP address (required)


            productId, str: Product ID (required)


            hostName, str: Host name (required)


            pkiEnabled, str: Device certificate provisioned (true) or not (false) (required)


            provisioningType, str: Provisioning type. Can be one of: new-device-adhoc, new-device-auto-provision, replacement-device (required)


            deviceType, str: Type of device. Currently only AP (Access Point) is supported (required)


            state, str: State of device. UNCLAIMED, IGNORED, PROVISIONED. Accepts a list of states (required)


            authStatus, str: Authentication status of device: authenticated, error, unsupported (required)


            startLastStateTransitionTime, str: Start of last state transition time (format: yyyy-MM-dd HH:mm:ss) (required)


            endLastStateTransitionTime, str: End of last state transition time (format: yyyy-MM-dd HH:mm:ss) (required)


            matchDeviceState, bool: Match device state. Boolean to match the given state (true) or not (false) (required)


            deviceMatchesARule, bool: Should device match a rule (true) or not (false) (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: CountResult
        """

        allParams = ['serialNumber', 'macAddress', 'ipAddress', 'productId', 'hostName', 'pkiEnabled', 'provisioningType', 'deviceType', 'state', 'authStatus', 'startLastStateTransitionTime', 'endLastStateTransitionTime', 'matchDeviceState', 'deviceMatchesARule', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getPnpDeviceCount" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/pnp-device/count'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('serialNumber' in params):
            queryParams['serialNumber'] = self.apiClient.toPathValue(params['serialNumber'])

        if ('macAddress' in params):
            queryParams['macAddress'] = self.apiClient.toPathValue(params['macAddress'])

        if ('ipAddress' in params):
            queryParams['ipAddress'] = self.apiClient.toPathValue(params['ipAddress'])

        if ('productId' in params):
            queryParams['productId'] = self.apiClient.toPathValue(params['productId'])

        if ('hostName' in params):
            queryParams['hostName'] = self.apiClient.toPathValue(params['hostName'])

        if ('pkiEnabled' in params):
            queryParams['pkiEnabled'] = self.apiClient.toPathValue(params['pkiEnabled'])

        if ('provisioningType' in params):
            queryParams['provisioningType'] = self.apiClient.toPathValue(params['provisioningType'])

        if ('deviceType' in params):
            queryParams['deviceType'] = self.apiClient.toPathValue(params['deviceType'])

        if ('state' in params):
            queryParams['state'] = self.apiClient.toPathValue(params['state'])

        if ('authStatus' in params):
            queryParams['authStatus'] = self.apiClient.toPathValue(params['authStatus'])

        if ('startLastStateTransitionTime' in params):
            queryParams['startLastStateTransitionTime'] = self.apiClient.toPathValue(params['startLastStateTransitionTime'])

        if ('endLastStateTransitionTime' in params):
            queryParams['endLastStateTransitionTime'] = self.apiClient.toPathValue(params['endLastStateTransitionTime'])

        if ('matchDeviceState' in params):
            queryParams['matchDeviceState'] = self.apiClient.toPathValue(params['matchDeviceState'])

        if ('deviceMatchesARule' in params):
            queryParams['deviceMatchesARule'] = self.apiClient.toPathValue(params['deviceMatchesARule'])



        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'CountResult')
        return responseObject




    def getPnpDeviceByDeviceId(self, **kwargs):
        """Retrieves a device with its ID

        Args:

            deviceId, str: Device ID (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: ZtdDeviceResult
        """

        allParams = ['deviceId', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getPnpDeviceByDeviceId" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/pnp-device/{deviceId}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('deviceId' in params):
            replacement = str(self.apiClient.toPathValue(params['deviceId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'deviceId' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'ZtdDeviceResult')
        return responseObject




    def deletePnpDeviceByDeviceId(self, **kwargs):
        """Deletes a device with its ID

        Args:

            deviceId, str: Device ID (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: TaskIdResult
        """

        allParams = ['deviceId', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method deletePnpDeviceByDeviceId" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/pnp-device/{deviceId}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'DELETE'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('deviceId' in params):
            replacement = str(self.apiClient.toPathValue(params['deviceId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'deviceId' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject









#!/usr/bin/env python
#pylint: skip-file
"""
Copyright 2016 Cisco Systems

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

class ZtdDevice(object):



    def __init__(self):
        """
        Attributes:
          swaggerTypes (dict): The key is attribute name and the value is attribute type.
          attributeMap (dict): The key is attribute name and the value is json key in definition.
        """
        self.swaggerTypes = {

            'state': 'str',


            'hostName': 'str',


            'serialNumber': 'str',


            'ipAddress': 'str',


            'id': 'str',


            'platformId': 'str',


            'site': 'str',


            'macAddress': 'str',


            'configId': 'str',


            'licenseString': 'str',


            'apCount': 'str',


            'isMobilityController': 'str',


            'pkiEnabled': 'bool',


            'sudiRequired': 'bool',


            'memberUdi': 'str',


            'imageId': 'str',


            'firstContact': 'str',


            'lastContact': 'str',


            'fileDestination': 'str',


            'backoffTimer': 'str',


            'provisioningType': 'str',


            'unclaimedHint': 'str',


            'bootstrapId': 'str',


            'oldNetworkDeviceId': 'str',


            'hwRevision': 'str',


            'mainMemSize': 'str',


            'boardId': 'str',


            'boardReworkId': 'str',


            'midplaneVersion': 'str',


            'versionString': 'str',


            'imageFile': 'str',


            'returnToRomReason': 'str',


            'bootVariable': 'str',


            'bootLdrVariable': 'str',


            'configVariable': 'str',


            'configReg': 'str',


            'configRegNext': 'str',


            'topologyInfo': 'str',


            'filesystemInfo': 'str',


            'deviceDetailsLastUpdate': 'str',


            'slotNumber': 'str',


            'certificateNeededState': 'str',


            'pnpProfileUsedAddr': 'str',


            'pnpProfileUsedHost': 'str',


            'authStatus': 'DeviceAuthState',


            'deviceCertificate': 'str',


            'deviceType': 'str',


            'lastStateTransitionTime': 'str',


            'stateDisplay': 'str',


            'versionCompatible': 'str',


            'cleanup': 'bool',


            'memberDetail': 'list[ZtdMemberDetail]',


            'eulaAccepted': 'bool',


            'licenseLevel': 'str'
        }

        self.attributeMap = {

            'state': 'state',

            'hostName': 'hostName',

            'serialNumber': 'serialNumber',

            'ipAddress': 'ipAddress',

            'id': 'id',

            'platformId': 'platformId',

            'site': 'site',

            'macAddress': 'macAddress',

            'configId': 'configId',

            'licenseString': 'licenseString',

            'apCount': 'apCount',

            'isMobilityController': 'isMobilityController',

            'pkiEnabled': 'pkiEnabled',

            'sudiRequired': 'sudiRequired',

            'memberUdi': 'memberUdi',

            'imageId': 'imageId',

            'firstContact': 'firstContact',

            'lastContact': 'lastContact',

            'fileDestination': 'fileDestination',

            'backoffTimer': 'backoffTimer',

            'provisioningType': 'provisioningType',

            'unclaimedHint': 'unclaimedHint',

            'bootstrapId': 'bootstrapId',

            'oldNetworkDeviceId': 'oldNetworkDeviceId',

            'hwRevision': 'hwRevision',

            'mainMemSize': 'mainMemSize',

            'boardId': 'boardId',

            'boardReworkId': 'boardReworkId',

            'midplaneVersion': 'midplaneVersion',

            'versionString': 'versionString',

            'imageFile': 'imageFile',

            'returnToRomReason': 'returnToRomReason',

            'bootVariable': 'bootVariable',

            'bootLdrVariable': 'bootLdrVariable',

            'configVariable': 'configVariable',

            'configReg': 'configReg',

            'configRegNext': 'configRegNext',

            'topologyInfo': 'topologyInfo',

            'filesystemInfo': 'filesystemInfo',

            'deviceDetailsLastUpdate': 'deviceDetailsLastUpdate',

            'slotNumber': 'slotNumber',

            'certificateNeededState': 'certificateNeededState',

            'pnpProfileUsedAddr': 'pnpProfileUsedAddr',

            'pnpProfileUsedHost': 'pnpProfileUsedHost',

            'authStatus': 'authStatus',

            'deviceCertificate': 'deviceCertificate',

            'deviceType': 'deviceType',

            'lastStateTransitionTime': 'lastStateTransitionTime',

            'stateDisplay': 'stateDisplay',

            'versionCompatible': 'versionCompatible',

            'cleanup': 'cleanup',

            'memberDetail': 'memberDetail',

            'eulaAccepted': 'eulaAccepted',

            'licenseLevel': 'licenseLevel'

        }


        #Device state

        self.state = None # str

        #Host name

        self.hostName = None # str

        #Serial number

        self.serialNumber = None # str


        self.ipAddress = None # str

        #Device ID

        self.id = None # str

        #Platform ID

        self.platformId = None # str

        #Project to which device belongs if pre-provisioned

        self.site = None # str


        self.macAddress = None # str

        #Configuration file ID

        self.configId = None # str

        #License information

        self.licenseString = None # str

        #Wireless AP count

        self.apCount = None # str

        #Specify if device is a wireless mobility controller

        self.isMobilityController = None # str

        #Configure PKCS#12 trust point during PnP workflow if true

        self.pkiEnabled = None # bool


        self.sudiRequired = None # bool

        #Unique device ID of redundant/stack switches

        self.memberUdi = None # str

        #Image file ID

        self.imageId = None # str

        #First contact time of device

        self.firstContact = None # str

        #Last contact time of device

        self.lastContact = None # str

        #Location on device to which image/config files will be copied

        self.fileDestination = None # str

        #Backoff timer

        self.backoffTimer = None # str

        #Type of device

        self.provisioningType = None # str

        #Hint whether device might be RMA

        self.unclaimedHint = None # str

        #Bootstrap file ID

        self.bootstrapId = None # str

        #Inventory device of which this device is the replacement

        self.oldNetworkDeviceId = None # str

        #HW revision

        self.hwRevision = None # str

        #Main memory size

        self.mainMemSize = None # str

        #Board ID

        self.boardId = None # str

        #Board rework ID

        self.boardReworkId = None # str

        #Mid plane version

        self.midplaneVersion = None # str

        #IOS Version

        self.versionString = None # str

        #Image ID

        self.imageFile = None # str

        #Return to rom reason

        self.returnToRomReason = None # str

        #Boot variable

        self.bootVariable = None # str

        #Boot ldr variable

        self.bootLdrVariable = None # str

        #Config variable

        self.configVariable = None # str

        #Config reg

        self.configReg = None # str

        #Config reg next

        self.configRegNext = None # str

        #Information about topology

        self.topologyInfo = None # str

        #Information about filesystem

        self.filesystemInfo = None # str

        #Timestamp when the device details were last read

        self.deviceDetailsLastUpdate = None # str

        #Slot number

        self.slotNumber = None # str

        #State which is detected as happening over http instead of https

        self.certificateNeededState = None # str

        #PnP server ipv4 address used by pnp profile

        self.pnpProfileUsedAddr = None # str

        #PnP server hostname used by pnp profile

        self.pnpProfileUsedHost = None # str


        self.authStatus = None # DeviceAuthState


        self.deviceCertificate = None # str


        self.deviceType = None # str

        #Last state transition time of device

        self.lastStateTransitionTime = None # str


        self.stateDisplay = None # str


        self.versionCompatible = None # str

        self.cleanup = None # bool

        self.memberDetail = None # list[ZtdMemberDetail]

        #CLI execution EULA accepted or not
        self.eulaAccepted = None # bool

        #CLI execution license level
        self.licenseLevel = None # str




#!/usr/bin/env python
#pylint: skip-file
"""
NetworkdeviceApi.py
    Copyright 2016 Cisco Systems

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
import sys
import os
import urllib.request, urllib.parse, urllib.error

from .models import *


class NetworkdeviceApi(object):

    def __init__(self, apiClient):
      self.apiClient = apiClient



    def getAllNetworkDevice(self, **kwargs):
        """Retrieves the network devices by filters

        Args:

            scope, str: Authorization Scope for RBAC (required)



        Returns: NetworkDeviceListResult
        """

        allParams = ['scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getAllNetworkDevice" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'NetworkDeviceListResult')
        return responseObject




    def updateNetworkDevice(self, **kwargs):
        """Updates network device role

        Args:

            networkDeviceBriefNIO, NetworkDeviceBriefNIO: networkDeviceBriefNIO (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: TaskIdResult
        """

        allParams = ['networkDeviceBriefNIO', 'scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method updateNetworkDevice" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/brief'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'PUT'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']







        if ('networkDeviceBriefNIO' in params):
            bodyParam = params['networkDeviceBriefNIO']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def getRunningConfig(self, **kwargs):
        """Retrieves device config list

        Args:

            scope, str: Authorization Scope for RBAC (required)



        Returns: RawCliInfoNIOListResult
        """

        allParams = ['scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getRunningConfig" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/config'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'RawCliInfoNIOListResult')
        return responseObject




    def getNetworkConfigCount(self, **kwargs):
        """Retrieves config count

        Args:

            scope, str: Authorization Scope for RBAC (required)



        Returns: CountResult
        """

        allParams = ['scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getNetworkConfigCount" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/config/count'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'CountResult')
        return responseObject




    def getNetworkDeviceCount(self, **kwargs):
        """Retrieves network device count by filters

        Args:

            scope, str: Authorization Scope for RBAC (required)



        Returns: CountResult
        """

        allParams = ['scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getNetworkDeviceCount" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/count'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'CountResult')
        return responseObject




    def getNetworkDeviceByIp(self, **kwargs):
        """Retrieves network device by IP address

        Args:

            ipAddress, str: Device IP address (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: NetworkDeviceResult
        """

        allParams = ['ipAddress', 'scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getNetworkDeviceByIp" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/ip-address/{ipAddress}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('ipAddress' in params):
            replacement = str(self.apiClient.toPathValue(params['ipAddress']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'ipAddress' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'NetworkDeviceResult')
        return responseObject




    def getNetworkDeviceLocation(self, **kwargs):
        """Retrieves device location

        Args:

            scope, str: Authorization Scope for RBAC (required)



        Returns: NetworkDeviceListResult
        """

        allParams = ['scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getNetworkDeviceLocation" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/location'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'NetworkDeviceListResult')
        return responseObject




    def addNetworkDeviceLocation(self, **kwargs):
        """Associates location with device

        Args:

            networkDeviceNIO, NetworkDeviceNIO: networkDeviceNIO (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: TaskIdResult
        """

        allParams = ['networkDeviceNIO', 'scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method addNetworkDeviceLocation" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/location'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'POST'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']







        if ('networkDeviceNIO' in params):
            bodyParam = params['networkDeviceNIO']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def getNetworkDeviceByLocationId(self, **kwargs):
        """Retrieves network device by location ID

        Args:

            locationId, str: Location ID (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: NetworkDeviceListResult
        """

        allParams = ['locationId', 'scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getNetworkDeviceByLocationId" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/location/{locationId}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('locationId' in params):
            replacement = str(self.apiClient.toPathValue(params['locationId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'locationId' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'NetworkDeviceListResult')
        return responseObject




    def getNetworkDeviceByLocationByRange(self, **kwargs):
        """Retrieves network devices with location by range

        Args:

            locationId, str: Location ID (required)


            startIndex, int: Start index (required)


            recordsToReturn, int: Number of records to return (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: NetworkDeviceListResult
        """

        allParams = ['locationId', 'startIndex', 'recordsToReturn', 'scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getNetworkDeviceByLocationByRange" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/location/{locationId}/{startIndex}/{recordsToReturn}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('locationId' in params):
            replacement = str(self.apiClient.toPathValue(params['locationId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'locationId' + '}',
                                                replacement)

        if ('startIndex' in params):
            replacement = str(self.apiClient.toPathValue(params['startIndex']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'startIndex' + '}',
                                                replacement)

        if ('recordsToReturn' in params):
            replacement = str(self.apiClient.toPathValue(params['recordsToReturn']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'recordsToReturn' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'NetworkDeviceListResult')
        return responseObject




    def getNetworkDeviceLocationByRange(self, **kwargs):
        """Retrieves device location range

        Args:

            startIndex, int: startIndex (required)


            recordsToReturn, int: recordsToReturn (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: NetworkDeviceListResult
        """

        allParams = ['startIndex', 'recordsToReturn', 'scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getNetworkDeviceLocationByRange" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/location/{startIndex}/{recordsToReturn}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('startIndex' in params):
            replacement = str(self.apiClient.toPathValue(params['startIndex']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'startIndex' + '}',
                                                replacement)

        if ('recordsToReturn' in params):
            replacement = str(self.apiClient.toPathValue(params['recordsToReturn']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'recordsToReturn' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'NetworkDeviceListResult')
        return responseObject




    def getNetworkManagementInfo(self, **kwargs):
        """Get the network management information

        Args:

            scope, str: Authorization Scope for RBAC (required)



        Returns: NetworkManagementInfoResult
        """

        allParams = ['scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getNetworkManagementInfo" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/management-info'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'NetworkManagementInfoResult')
        return responseObject




    def getNetworkManagementInfoCount(self, **kwargs):
        """Get the network management information count

        Args:

            scope, str: Authorization Scope for RBAC (required)



        Returns: CountResult
        """

        allParams = ['scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getNetworkManagementInfoCount" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/management-info/count'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'CountResult')
        return responseObject




    def getNetworkDeviceBySerialNumber(self, **kwargs):
        """Retrieves network device by serial number

        Args:

            serialNumber, str: Device serial number (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: NetworkDeviceResult
        """

        allParams = ['serialNumber', 'scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getNetworkDeviceBySerialNumber" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/serial-number/{serialNumber}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('serialNumber' in params):
            replacement = str(self.apiClient.toPathValue(params['serialNumber']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'serialNumber' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'NetworkDeviceResult')
        return responseObject




    def getNetworkDeviceById(self, **kwargs):
        """Retrieves network device by ID

        Args:

            id, str: Device ID (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: NetworkDeviceResult
        """

        allParams = ['id', 'scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getNetworkDeviceById" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/{id}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('id' in params):
            replacement = str(self.apiClient.toPathValue(params['id']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'id' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'NetworkDeviceResult')
        return responseObject




    def deleteDevicebyId(self, **kwargs):
        """Delete network device by ID

        Args:

            id, str: Device ID (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: TaskIdResult
        """

        allParams = ['id', 'scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method deleteDevicebyId" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/{id}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'DELETE'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('id' in params):
            replacement = str(self.apiClient.toPathValue(params['id']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'id' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def getNetworkDeviceBrief(self, **kwargs):
        """Retrieves network device brief by ID

        Args:

            id, str: Device ID (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: NetworkDeviceBriefNIOResult
        """

        allParams = ['id', 'scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getNetworkDeviceBrief" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/{id}/brief'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('id' in params):
            replacement = str(self.apiClient.toPathValue(params['id']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'id' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'NetworkDeviceBriefNIOResult')
        return responseObject




    def getNetworkDeviceLocationById(self, **kwargs):
        """Retrieves device location by device ID

        Args:

            id, str: Device ID (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: NetworkDeviceResult
        """

        allParams = ['id', 'scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getNetworkDeviceLocationById" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/{id}/location'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('id' in params):
            replacement = str(self.apiClient.toPathValue(params['id']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'id' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'NetworkDeviceResult')
        return responseObject




    def deleteNetworkLocationById(self, **kwargs):
        """Removes network device location

        Args:

            id, str: Device ID (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: TaskIdResult
        """

        allParams = ['id', 'scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method deleteNetworkLocationById" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/{id}/location'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'DELETE'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('id' in params):
            replacement = str(self.apiClient.toPathValue(params['id']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'id' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject



    #changed this from getRunningConfig to getRunningConfigById manually
    def getRunningConfigById(self, **kwargs):
        """Retrieves device config by device id

        Args:

            networkDeviceId, str: networkDeviceId (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: SuccessResult
        """

        allParams = ['networkDeviceId', 'scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getRunningConfig" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/{networkDeviceId}/config'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('networkDeviceId' in params):
            replacement = str(self.apiClient.toPathValue(params['networkDeviceId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'networkDeviceId' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'SuccessResult')
        return responseObject




    def getNetworkDeviceRange(self, **kwargs):
        """Retrieves network device by range

        Args:

            startIndex, int: Start index (required)


            recordsToReturn, int: Number of records to return (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: NetworkDeviceListResult
        """

        allParams = ['startIndex', 'recordsToReturn', 'scope']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getNetworkDeviceRange" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/network-device/{startIndex}/{recordsToReturn}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('startIndex' in params):
            replacement = str(self.apiClient.toPathValue(params['startIndex']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'startIndex' + '}',
                                                replacement)

        if ('recordsToReturn' in params):
            replacement = str(self.apiClient.toPathValue(params['recordsToReturn']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'recordsToReturn' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'NetworkDeviceListResult')
        return responseObject









"""
Copyright 2016 Cisco Systems

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import json

try:
    from uniq.common import logger
except:
    import logging
    logger = logging.getLogger("uniq")
from .exceptions import ApiClientException
from .exceptions import ConnectionException

import requests


class ClientManager(object):
    """ Client manager to interact with API Clients of various services. """

    def __init__(self, server, username, password, base_url, protocol="https", port=None):
        """ Object initializer

        Initializer also authenticates using the credentials, and stores the generated
        authentication ticket/token.

        Args:
            server (str): cluster server name (routable DNS addess or ip)
            username (str): user name to authenticate with
            password (str): password to authenticate with
            port (str): port number
            service (str): service to which the api calls are to be made
            connect (bool): flag to authenticate and establish api client
        """

        self.log = logger

        self.server = server
        self.username = username
        self.password = password

        self.port = str(port)
        if port:
            self.server = self.server + ":" + self.port

        if protocol not in ["http", "https"]:
            self.log.error("Not supported protocol {}.".format(protocol))
            raise ApiClientException("Not supported protocol {}.".format(protocol))

        self.base_url = "{}://{}{}".format(protocol, self.server, base_url)

        self._default_headers = {}
        self._common_headers = {}

    def __repr__(self):
        """ Overrides the default object representation to display the object attributes. """

        return "[API Client: <server:{}> <username:{}> <password:{}>]".format(self.server,
                                                                             self.username,
                                                                             self.password)

    def add_api(self, name, obj):
        """ Add an api client to client manager.

        Args:
            name (str): name you want to set to the api client, has to follow python variable naming
                        rule.
            obj (object): api client which actually calling call_api method.
        """

        setattr(self, name, obj)

    def connect(self):
        """ Generates a new ticket/token. """

        self.log.info("Connecting to the API client.")
        self.authenticate()

    def disconnect(self):
        """ Disconnect from API client"""

        self.log.info("Disconnecting from the API client.")

    def authenticate(self):
        """ Generates a new authentication ticket or token. """

        raise NotImplementedError

    @property
    def default_headers(self):
        return self._default_headers

    @default_headers.setter
    def default_headers(self, headers):
        """ Set default headers of client.

        Args:
            headers (dict): headers to set.
        """

        self._default_headers.update(headers)

    @property
    def common_headers(self):
        return self._common_headers

    @common_headers.setter
    def common_headers(self, headers):
        """ Set common headers of client.

        Args:
            headers (dict): headers to set.
        """

        self._common_headers.update(headers)

    def call_api(self, method, resource_path, raise_exception=True, response_dict=True, **kwargs):
        """ Handles the requests and response.

        Args:
            method (str): type of request.
            resource_path (str): URL in the request object.
            raise_exception (boolean): If True, http exceptions will be raised.
            response_dict (boolean): If True, response dict is returned, else response object
            url (optional): URL for the new Request object.
            params (optional): Dictionary or bytes to be sent in the query string for the Request.
            data (optional): Dictionary, bytes, or file-like object to send in the body of the
                             Request.
            json (optional): json data to send in the body of the Request.
            headers (optional): Dictionary of HTTP Headers to send with the Request.
            cookies (optional): Dict or CookieJar object to send with the Request.
            files (optional): Dictionary of 'name': file-like-objects
                              (or {'name': ('filename', fileobj)}) for multipart encoding upload.
            auth (optional): Auth tuple to enable Basic/Digest/Custom HTTP Auth.
            timeout (float or tuple) (optional): How long to wait for the server to send data
                                                 before giving up, as a float, or a (connect
                                                 timeout, read timeout) tuple.
            allow_redirects (bool) (optional): Boolean. Set to True if POST/PUT/DELETE redirect
                                               following is allowed.
            proxies (optional): Dictionary mapping protocol to the URL of the proxy.
            verify (optional): if True, the SSL cert will be verified. A CA_BUNDLE path can also
                               be provided.
            stream (optional): if False, the response content will be immediately downloaded.
            cert (optional): if String, path to ssl client cert file (.pem). If Tuple,
                             (‘cert’, ‘key’) pair
        Returns:
            response object or response dict depending on boolean variable - response_dict.
        """

        url = self.base_url + resource_path

        if "/api/v1/v2/" in url:
            url = url.replace("/api/v1/v2/", "/api/v2/")

        if "headers" in kwargs:
            headers = kwargs.pop("headers")
        else:
            headers = self.default_headers

        headers.update(self.common_headers)

        if "verify" in kwargs:
            verify = kwargs.pop("verify")
        else:
            verify = False
            requests.packages.urllib3.disable_warnings()

        self.log.debug("Request:\nmethod:\n{}\nurl: {}\nheaders: {}\nParameters: {}".format(
                method, url, headers, kwargs))
        response = requests.request(method, url, headers=headers, verify=verify, **kwargs)
        self.log.debug("Response: {}".format(response.text))

        if raise_exception:
            try:
                response.raise_for_status()
            except requests.exceptions.HTTPError as e:
                self.log.exception("Error Code: {} URL:{} Data:{} Headers:{} Message:{}"
                                   .format(e.response.status_code, url, kwargs, headers,
                                           e.response.text))
                raise e

        if response_dict:
            return json.loads(response.text)
        else:
            return response



#!/usr/bin/env python
#pylint: skip-file
"""
TicketApi.py
    Copyright 2016 Cisco Systems

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
import sys
import os
import urllib.request, urllib.parse, urllib.error

from .models import *


class TicketApi(object):

    def __init__(self, apiClient):
      self.apiClient = apiClient



    def addTicket(self, **kwargs):
        """addTicket

        Args:

            user, User: user (required)



        Returns: TicketRbacResult
        """

        allParams = ['user']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method addTicket" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/ticket'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'POST'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'










        if ('user' in params):
            bodyParam = params['user']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TicketRbacResult')
        return responseObject




    def createTicketAttribute(self, **kwargs):
        """createTicketAttribute

        Args:

            ticketAttribute, TicketAttribute: ticketAttribute (required)



        Returns: SuccessResult
        """

        allParams = ['ticketAttribute']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method createTicketAttribute" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/ticket/attribute'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'POST'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'










        if ('ticketAttribute' in params):
            bodyParam = params['ticketAttribute']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'SuccessResult')
        return responseObject




    def getIdleTimeout(self, **kwargs):
        """getIdleTimeout

        Args:


        Returns: TicketAttributeResult
        """

        allParams = []

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getIdleTimeout" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/ticket/attribute/idletimeout'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'











        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TicketAttributeResult')
        return responseObject




    def getSessionTimeout(self, **kwargs):
        """getSessionTimeout

        Args:


        Returns: TicketAttributeResult
        """

        allParams = []

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getSessionTimeout" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/ticket/attribute/sessiontimeout'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'











        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TicketAttributeResult')
        return responseObject




    def deleteTicketAttribute(self, **kwargs):
        """deleteTicketAttribute

        Args:

            attribute, str: attribute (required)



        Returns: SuccessResult
        """

        allParams = ['attribute']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method deleteTicketAttribute" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/ticket/attribute/{attribute}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'DELETE'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'






        if ('attribute' in params):
            replacement = str(self.apiClient.toPathValue(params['attribute']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'attribute' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'SuccessResult')
        return responseObject




    def deleteTicket(self, **kwargs):
        """deleteTicket

        Args:

            ticket, str: ticket (required)



        Returns: SuccessResult
        """

        allParams = ['ticket']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method deleteTicket" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/ticket/{ticket}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'DELETE'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'






        if ('ticket' in params):
            replacement = str(self.apiClient.toPathValue(params['ticket']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'ticket' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'SuccessResult')
        return responseObject









"""
Copyright 2016 Cisco Systems

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import time
import json
from http.client import NOT_FOUND

from requests.exceptions import HTTPError

from uniq.apis.nb.services.services import Services
from uniq.apis.exceptions import ApiClientException

class Task(Services):
    """ Utility to use Task service. """

    POLL_FREQUENCY = 3  # seconds
    TASK_DEFAULT_TIMEOUT = 60  # seconds

    GET_TASK_MAX_RETRIES = 6
    EXPONENTIAL_BACKOFF_MULTIPLIER = 2

    def __wait_for_task_complete(self, task_id, timeout=TASK_DEFAULT_TIMEOUT,
                                 poll_frequency=POLL_FREQUENCY):
        """ Waits for task complete.

        Args:
            task_id (str): Id of the task.
            timeout (float): timeout in seconds.
            poll_frequency (float): poll frequency in seconds of getting task status

        Returns:
            the failure/success response of the task.
        """

        def get_task_status(task_id):
            """ Get task status.

            Args:
                task_id (str): id of task to check.

            Returns:
                task_response(AugmentedTaskDTO) if task is complete, else return False.
            """

            task_response = self.__get_task_response(task_id)
            if self.__is_task_success(task_response) or self.__is_task_failed(task_response):
                return task_response
            else:
                return False

        task_response = self.utils.wait.until(
            get_task_status, task_id=task_id, timeout=timeout, interval=poll_frequency,
            message="Task didn't finish in {} seconds.".format(timeout))
        return task_response

    def __wait_for_task_success(self, task_id=None, timeout=TASK_DEFAULT_TIMEOUT,
                                poll_frequency=POLL_FREQUENCY):
        """ Waits for task to be success for a given task_id.

        Args:
            task_id (str): Id of the task.
            timeout (float): timeout in seconds.
            poll_frequency (float): poll frequency in seconds of getting task status

        Returns:
            the failure/success response of the task.
        """

        task_response = self.__wait_for_task_complete(task_id, timeout, poll_frequency)

        if self.__is_task_success(task_response):
            return task_response
        elif self.__is_task_failed(task_response):
            raise ApiClientException(
                "Task Failed. errorCode: {}, failure reason: {}, progress: {}".format(
                    task_response.errorCode, task_response.failureReason, task_response.progress))

    def __wait_for_task_failure(self, task_id=None, timeout=TASK_DEFAULT_TIMEOUT,
                                poll_frequency=POLL_FREQUENCY):
        """ Waits for the task to be failure for a given task_id

        Args:
            task_id (str): task_id is waiting for the failure status.
            timeout (int): time_out value to wait for failure status of the task.
            poll_frequency (float): poll frequency in seconds of getting task status.

        Returns:
            the failure/success response of the task.
        """

        task_response = self.__wait_for_task_complete(task_id, timeout, poll_frequency)
        if self.__is_task_failed(task_response):
            return task_response
        elif self.__is_task_success(task_response):
            raise ApiClientException("Task expected to fail, but actually succeed.")

    def wait_for_task_success(self, task_id_result, timeout=TASK_DEFAULT_TIMEOUT,
                              poll_frequency=POLL_FREQUENCY):
        """ Waits for task to be success for a given task_id.

        Args:
            task_id_result (TaskIdResult): TaskIdResult object is waiting for the failure status.
            timeout (float): timeout (in seconds) value to wait for failure status of the task.
            poll_frequency (float): poll frequency in seconds of getting task status.

        Returns:
            the failure/success response of the task.
        """

        task_id = self.get_task_id_from_task_id_result(task_id_result)
        return self.__wait_for_task_success(task_id=task_id, timeout=timeout,
                                            poll_frequency=poll_frequency)

    def wait_for_task_failure(self, task_id_result, timeout=TASK_DEFAULT_TIMEOUT,
                              poll_frequency=POLL_FREQUENCY):
        """ Waits for the task to be failure for a given task_id

        Args:
            task_id_result (TaskIdResult): TaskIdResult object is waiting for the failure status.
            timeout (float): timeout (in seconds) value to wait for failure status of the task.
            poll_frequency (float): poll frequency in seconds of getting task status.

        Returns:
            the failure/success response of the task.
        """

        task_id = self.get_task_id_from_task_id_result(task_id_result)
        return self.__wait_for_task_failure(task_id=task_id, timeout=timeout,
                                            poll_frequency=poll_frequency)

    def wait_for_task_complete(self, task_id_result, timeout=TASK_DEFAULT_TIMEOUT,
                               poll_frequency=POLL_FREQUENCY):
        """ Waits for task complete.

        Args:
            task_id (str): Id of the task.
            timeout (float): timeout in seconds.
            poll_frequency (float): poll frequency in seconds of getting task status.

        Returns:
            the failure/success response of the task.
        """

        task_id = self.get_task_id_from_task_id_result(task_id_result)
        return self.__wait_for_task_complete(task_id=task_id, timeout=timeout,
                                             poll_frequency=poll_frequency)

    def __get_task_response(self, task_id):
        """ Get task response by task id.

        Args:
            task_id (str): id of the task.

        Returns:
            task_response (AugmentedTaskDTO) of the task.
        """

        retry_interval = 1
        retries = self.GET_TASK_MAX_RETRIES
        for retry in range(retries):
            try:
                task_result = self.nb_api.task.getTask(taskId=task_id)
                task_response = task_result.response
                return task_response
            except HTTPError as err:
                error_code = err.response.status_code
                error_result = err.response._content.decode()
                if error_code == NOT_FOUND:
                    if retry < retries - 1:
                        time.sleep(retry_interval)
                        retry_interval *= self.EXPONENTIAL_BACKOFF_MULTIPLIER
                    else:
                        raise ApiClientException(
                            "Max retries ({0}) exceeded\nHTTP error code: {1}\nerror result: {2}"
                            .format(retries, error_code, json.loads(error_result)))
                else:
                    raise ApiClientException("HTTP error code: {0}\nerror result: {1}"
                                             .format(error_code, json.loads(error_result)))
        raise ApiClientException("Didn't get any valid task response for '{}'".format(task_id))

    def __is_task_failed(self, task_response):
        """ Returns true if task failed.

        Args:
            task_response(AugmentedTaskDTO): object of task response.
        """

        return task_response.isError is True

    def __is_task_success(self, task_response, error_codes=None):
        """ Returns true if task succeed.

        Args:
            task_response(AugmentedTaskDTO): object of task response.
            error_codes (list[str]): error codes to treat as success.

        Returns:
            true if task succeed.
        """

        if error_codes and hasattr(task_response, 'errorCode'):
            return task_response.errorCode in error_codes

        return not (task_response.isError or task_response.endTime is None)

    def get_task_id_from_task_id_result(self, task_id_result):
        """ Gets task id from TaskIdResult object.

        Args:
            task_id_result (TaskIdResult): TaskIdResult object.

        Returns:
            task_id (str).
        """

        return task_id_result.response.taskId

    def get_task_progress(self, task_id_result):
        """ Gives the progress of the task from task response

        Args:
            task_id_result (TaskIdResult): TaskIdResult object

        Returns:
            progress (str): the progress (current state) of the task
        """

        task_id = self.get_task_id_from_task_id_result(task_id_result)
        task_dto_response = self.__get_task_response(task_id)
        return task_dto_response.progress

    def get_task_tree(self, task_id, timeout=TASK_DEFAULT_TIMEOUT, poll_frequency=POLL_FREQUENCY):
        """ Returns the task objects of the parent and child task

        Args:
            task_id (str): task id
            timeout (int) : default wait time

        Returns:
            task_list_response (TaskDTOListResponse object): list with parent and child task objects
        """

        def task_tree_results(task_id):
            """ Get task tree results.

            Args:
                task_id (str): task id.

            Returns:
                task_list_response (TaskDTOListResponse) if found 2 or more tasks, else return False
            """

            task_list_response = self.nb_api.task.getTaskTree(taskId=task_id).response
            if len(task_list_response) > 1:
                return task_list_response
            else:
                return False

        return self.utils.wait.until(
            task_tree_results,
            task_id=task_id,
            timeout=timeout,
            interval=poll_frequency)

    def is_task_tree_success(self, task_id_result):
        """ Returns true if all tasks in task tree are successful.

        Args:
            task_id_result (TaskIdResult): TaskIdResult object to check.

        Returns:
            True if all tasks in stask tree succeed, if any task failed, return False.
        """

        task_response = self.wait_for_task_complete(task_id_result=task_id_result)
        if self.__is_task_failed(task_response):
            return False
        task_id = self.get_task_id_from_task_id_result(task_id_result=task_id_result)
        task_list_response = self.nb_api.task.getTaskTree(taskId=task_id).response

        for task in task_list_response:
            if self.__is_task_failed(task):
                return False
        return True

    def is_task_success(self, task_id_result=None, error_codes=None):
        """ Returns true if task succeed.

        Args:
            task_id_result (TaskIdResult): TaskIdResult object to check.
            error_codes (list[str]): error_codes considered as success.

        Returns:
            True if task succeed.
        """

        task_response = self.wait_for_task_complete(task_id_result=task_id_result)
        return self.__is_task_success(task_response=task_response, error_codes=error_codes)

    def is_task_failure(self, task_id_result=None):
        """ Returns true if task failed.

        Args:
            task_id_result (TaskIdResult): TaskIdResult object to check.

        Returns:
            True if task failed.
        """

        task_response = self.wait_for_task_complete(task_id_result=task_id_result)
        return self.__is_task_failed(task_response=task_response)

    def get_task_failure_reasons(self, task_id_result):
        """ Retrieves the task failure reasons.

        Args:
            task_id_result (TaskIdResult): TaskIdResult object to check.

        Returns:
            string containing a list of task failure reasons
        """

        task_id = self.get_task_id_from_task_id_result(task_id_result)
        task_list_response = self.get_task_tree(task_id=task_id)
        failure_reasons = []
        for task in task_list_response:
            if task.failureReason is not None:
                failure_reasons.append(task.failureReason)

        return ", ".join(failure_reasons)

    def get_task_response(self, task_id_result):
        """ Get current task response.

        Args:
            task_id_result (TaskIdResult): TaskIdResult object to check.

        Returns:
            task_response (AugmentedTaskDTO) of the task.
        """

        task_id = self.get_task_id_from_task_id_result(task_id_result)
        return self.__get_task_response(task_id)



"""
Copyright 2016 Cisco Systems

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import inspect

from uniq.apis.exceptions import UniqException


class ApiClient(object):
    """ Api client class to handle multiple version of apis. """

    def __init__(self, api_client, version_client_class_map, default_version,
                 similar_api_map_list=None):
        """ Initializer of ApiClient.

        Args:
            api_client (ClientManager): ClientManager instance.
            version_client_class_map (dict): key is version number, value is class of api client.
            default_version (str): default version.
            similar_api_map_list (list): list of map of apis with different name but same feature.
                                         Example: [{'v1': 'get', 'v2': 'getById'}]
        """

        self.version_client_class_map = version_client_class_map
        self.api_client = api_client
        self.apiClient = api_client # Depreciated, used for backward compatible.
        self.version_clients_map = {}
        self.versioned_apis = {}
        self.all_apis = {}
        self.default_version = default_version

        self._collect_apis(similar_api_map_list)

    def __getattr__(self, name):
        """ Get attribute based on name.

        Args:
            name (str): name of attribute.

        Returns:
            Wrapped api method(function) with additional argument api_verion.
        """

        if name in self.versioned_apis:
            return self.versioned_apis[name]
        raise AttributeError

    def _collect_apis(self, similar_api_map_list=None):
        """ Collect all apis and make them accessible by name and version.

        This method will collect all apis from clients of different versions, deal with apis
        with different name but same feature, and wrap all api methods so that we can choose
        different version api call with argument 'api_version'.

        Args:
            similar_api_map_list (dict): list of map of apis with different name but same feature.
                                         Example: [{'v1': 'get', 'v2': 'getById'}]
        """

        self.all_apis = {}
        for version, client_class in self.version_client_class_map.items():
            self.version_clients_map[version] = client_class(self.api_client)

        for version, client in self.version_clients_map.items():
            # Get all methods (api definition) in the api client
            apis = inspect.getmembers(client, predicate=inspect.ismethod)
            for name, method in apis:
                if name != '__init__':
                    if name not in self.all_apis:
                        self.all_apis[name] = {}
                    self.all_apis[name][version] = method

        if similar_api_map_list:
            for similar_api_map in similar_api_map_list:
                for version_1, name_1 in similar_api_map.items():
                    for version_2, name_2 in similar_api_map.items():
                        if name_1 != name_2 and version_2 not in self.all_apis[name_1]:
                            self.all_apis[name_1][version_2] = self.all_apis[name_2][version_2]

        for api_name, version_function_map in self.all_apis.items():
            self.versioned_apis[api_name] = self._add_api_version_as_argument(version_function_map)

    def _add_api_version_as_argument(self, version_function_map):
        """ Add api_version as an argument to all api method.

        Args:
            version_function_map (dict): key is version and value is function object.

        Returns:
            wrapper function which will decide which api to be call according to the version.
        """

        def wrapper(*args, **kwargs):
            """ Wrapper function.

                If api_version is set, return corresponding method.
                If not, and if there is only one method, return that method.
                if there are more than one method, return the default version one.
            """

            if 'api_version' in kwargs:
                api_version = kwargs.pop('api_version')
                if api_version in version_function_map:
                    return version_function_map[api_version](*args, **kwargs)
                else:
                    raise UniqException("endpoint '{}' doesn't support version '{}'"
                                        .format(list(version_function_map.values())[0].__name__,
                                                api_version))

            elif len(version_function_map) == 1:
                return list(version_function_map.values())[0](*args, **kwargs)
            else:
                return version_function_map[self.default_version](*args, **kwargs)
        return wrapper




"""
Copyright 2016 Cisco Systems

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import logging
logger = logging.getLogger("uniq")

class UniqException(Exception):
    """ Base exception class in uniq takes an error message to get initialized

    It makes a log entry of level exception using the error message used while calling
    UniqException. The log entry has a tag of [UniqException], for the purpose of
    easy filtering.

    UniqException can be directly raised as an exception as well.

    Usage:
        raise UniqException("error_message")
    """

    # Class variable to keep count of the number of times the UniqException is raised in
    # the life of a test case.
    __COUNTER = 0

    __EXTRA_ACTIONS = set()

    def __init__(self, error_message):
        """ Initializes the test exception instance. """

        super(UniqException, self).__init__()
        self.__class__.__COUNTER += 1
        logger.error("[{0}] {1}".format(self.__class__.__name__, error_message))
        self.error_message = error_message
        for action in self.__class__.__EXTRA_ACTIONS:
            action()

    def __str__(self):
        return self.error_message

class ApiClientException(UniqException):
    """ Exception class for API specific scenarios"""

    def __init__(self, message=None):
        super(ApiClientException, self).__init__(message)

    @staticmethod
    def raise_exception(message=None, code=None):
        """ Raises exception based on the error code

        Args:
            message(str): response
            code (int): status code of the response
        """

        if code == 400:
            #TODO (hikoppu): Add more expections if any, which raise 404
            raise InvalidContentException(message)
        elif code == 401:
            raise UnauthorizedException(message)
        elif code == 404:
            #TODO (hikoppu): Add more expections if any, which raise 404
            raise ApiNotFoundException(message)

class ConnectionException(ApiClientException):
    def __init__(self, message=None):
        super(ConnectionException, self).__init__(
            message if message else "Connection to the server failed.")

class InvalidContentException(ApiClientException):
    def __init__(self, message=None):
        super(InvalidContentException, self).__init__(
            message if message else "Invalid content in request. ")

class UnauthorizedException(ApiClientException):
    def __init__(self, message=None):
        super(UnauthorizedException, self).__init__(
            message if message else "Unable to autheticate.")

class ApiNotFoundException(ApiClientException):
    def __init__(self, message=None):
        super(ApiNotFoundException, self).__init__(
            message if message else "Connection to the server failed.")









#!/usr/bin/env python
#pylint: skip-file
"""
Copyright 2016 Cisco Systems

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

class FileObject(object):



    def __init__(self):
        """
        Attributes:
          swaggerTypes (dict): The key is attribute name and the value is attribute type.
          attributeMap (dict): The key is attribute name and the value is json key in definition.
        """
        self.swaggerTypes = {

            'nameSpace': 'str',


            'id': 'str',


            'downloadPath': 'str',


            'fileFormat': 'str',


            'fileSize': 'str',


            'md5Checksum': 'str',


            'sha1Checksum': 'str',


            'name': 'str',


            'attributeInfo': 'object'

        }

        self.attributeMap = {

            'nameSpace': 'nameSpace',

            'id': 'id',

            'downloadPath': 'downloadPath',

            'fileFormat': 'fileFormat',

            'fileSize': 'fileSize',

            'md5Checksum': 'md5Checksum',

            'sha1Checksum': 'sha1Checksum',

            'name': 'name',

            'attributeInfo': 'attributeInfo'

        }


        #A group of file IDs contained in a common nameSpace

        self.nameSpace = None # str

        #file indentification number

        self.id = None # str

        #Absolute path of the file

        self.downloadPath = None # str

        #MIME Type of the File. e.g. text/plain, application/xml, audio/mpeg

        self.fileFormat = None # str

        #Size of the file in bytes

        self.fileSize = None # str

        #md5Checksum of the file

        self.md5Checksum = None # str

        #sha1Checksum of the file

        self.sha1Checksum = None # str

        #Name of the file

        self.name = None # str


        self.attributeInfo = None # object

